@@ -18,25 +18,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <string.h>
-#include <glib/gfileutils.h>
-#include <glib/gi18n.h>
-#include <glib/gmacros.h>
-#include <glib/gmessages.h>
-#include <glib/gslist.h>
-#include <glib/gstrfuncs.h>
-#include <glib-object.h>
 #include <gmodule.h>
 
-#include <gdk/gdkenumtypes.h>
-#include <gdk/gdkkeys.h>
-#include <gtk/gtktypeutils.h>
 #include "gtkbuilderprivate.h"
 #include "gtkbuilder.h"
 #include "gtkbuildable.h"
 #include "gtkdebug.h"
-#include "gtktypeutils.h"
-#include "gtkalias.h"
+#include "gtkversion.h"
+#include "gtktypebuiltins.h"
+#include "gtkintl.h"
+
 
 static void free_property_info (PropertyInfo *info);
 static void free_object_info (ObjectInfo *info);
@@ -139,24 +133,6 @@ error_invalid_tag (ParserData *data,
 		 line_number, char_number, tag);
 }
 
-static void
-error_missing_property_value (ParserData *data,
-			      GError **error)
-{
-  gint line_number, char_number;
-
-  g_markup_parse_context_get_position (data->ctx,
-                                       &line_number,
-                                       &char_number);
-
-  g_set_error (error,
-	       GTK_BUILDER_ERROR,
-	       GTK_BUILDER_ERROR_MISSING_PROPERTY_VALUE,
-	       "%s:%d:%d <property> must have a value set",
-	       data->filename,
-	       line_number, char_number);
-}
-
 gboolean
 _gtk_builder_boolean_from_string (const gchar  *string,
 				  gboolean     *value,
@@ -204,8 +180,9 @@ _gtk_builder_boolean_from_string (const gchar  *string,
 }
 
 static GObject *
-builder_construct (ParserData *data,
-                   ObjectInfo *object_info)
+builder_construct (ParserData  *data,
+                   ObjectInfo  *object_info,
+		   GError     **error)
 {
   GObject *object;
 
@@ -216,7 +193,10 @@ builder_construct (ParserData *data,
 
   object_info->properties = g_slist_reverse (object_info->properties);
 
-  object = _gtk_builder_construct (data->builder, object_info);
+  object = _gtk_builder_construct (data->builder, object_info, error);
+  if (!object)
+    return NULL;
+
   g_assert (G_IS_OBJECT (object));
 
   object_info->object = object;
@@ -245,11 +225,84 @@ _get_type_by_symbol (const gchar* symbol)
 }
 
 static void
-parse_object (ParserData   *data,
-              const gchar  *element_name,
-              const gchar **names,
-              const gchar **values,
-              GError      **error)
+parse_requires (ParserData   *data,
+		const gchar  *element_name,
+		const gchar **names,
+		const gchar **values,
+		GError      **error)
+{
+  RequiresInfo *req_info;
+  const gchar  *library = NULL;
+  const gchar  *version = NULL;
+  gchar       **split;
+  gint          i, version_major = 0, version_minor = 0;
+  gint          line_number, char_number;
+
+  g_markup_parse_context_get_position (data->ctx,
+                                       &line_number,
+                                       &char_number);
+
+  for (i = 0; names[i] != NULL; i++)
+    {
+      if (strcmp (names[i], "lib") == 0)
+        library = values[i];
+      else if (strcmp (names[i], "version") == 0)
+	version = values[i];
+      else
+	error_invalid_attribute (data, element_name, names[i], error);
+    }
+
+  if (!library || !version)
+    {
+      error_missing_attribute (data, element_name, 
+			       version ? "lib" : "version", error);
+      return;
+    }
+
+  if (!(split = g_strsplit (version, ".", 2)) || !split[0] || !split[1])
+    {
+      g_set_error (error,
+		   GTK_BUILDER_ERROR,
+		   GTK_BUILDER_ERROR_INVALID_VALUE,
+		   "%s:%d:%d <%s> attribute has malformed value \"%s\"",
+		   data->filename,
+		   line_number, char_number, "version", version);
+      return;
+    }
+  version_major = g_ascii_strtoll (split[0], NULL, 10);
+  version_minor = g_ascii_strtoll (split[1], NULL, 10);
+  g_strfreev (split);
+
+  req_info = g_slice_new0 (RequiresInfo);
+  req_info->library = g_strdup (library);
+  req_info->major   = version_major;
+  req_info->minor   = version_minor;
+  state_push (data, req_info);
+  req_info->tag.name = element_name;
+}
+
+static gboolean
+is_requested_object (const gchar *object,
+                     ParserData  *data)
+{
+  GSList *l;
+
+  for (l = data->requested_objects; l; l = l->next)
+    {
+      if (strcmp (l->data, object) == 0)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+parse_object (GMarkupParseContext  *context,
+              ParserData           *data,
+              const gchar          *element_name,
+              const gchar         **names,
+              const gchar         **values,
+              GError              **error)
 {
   ObjectInfo *object_info;
   ChildInfo* child_info;
@@ -257,13 +310,12 @@ parse_object (ParserData   *data,
   gchar *object_class = NULL;
   gchar *object_id = NULL;
   gchar *constructor = NULL;
+  gint line, line2;
 
   child_info = state_peek_info (data, ChildInfo);
   if (child_info && strcmp (child_info->tag.name, "object") == 0)
     {
       error_invalid_tag (data, element_name, NULL, error);
-      if (child_info)
-	free_object_info ((ObjectInfo*)child_info);
       return;
     }
 
@@ -284,10 +336,11 @@ parse_object (ParserData   *data,
           object_class = _get_type_by_symbol (values[i]);
           if (!object_class)
             {
-              g_set_error (error, GTK_BUILDER_ERROR, 
+              g_markup_parse_context_get_position (context, &line, NULL);
+              g_set_error (error, GTK_BUILDER_ERROR,
                            GTK_BUILDER_ERROR_INVALID_TYPE_FUNCTION,
-                           _("Invalid type function: `%s'"),
-                           values[i]);
+                           _("Invalid type function on line %d: '%s'"),
+                           line, values[i]);
               return;
             }
         }
@@ -310,16 +363,53 @@ parse_object (ParserData   *data,
       return;
     }
 
+  ++data->cur_object_level;
+
+  /* check if we reached a requested object (if it is specified) */
+  if (data->requested_objects && !data->inside_requested_object)
+    {
+      if (is_requested_object (object_id, data))
+        {
+          data->requested_object_level = data->cur_object_level;
+
+          GTK_NOTE (BUILDER, g_print ("requested object \"%s\" found at level %d\n",
+                                      object_id,
+                                      data->requested_object_level));
+
+          data->inside_requested_object = TRUE;
+        }
+      else
+        {
+          g_free (object_class);
+          g_free (object_id);
+          g_free (constructor);
+          return;
+        }
+    }
+
   object_info = g_slice_new0 (ObjectInfo);
   object_info->class_name = object_class;
   object_info->id = object_id;
   object_info->constructor = constructor;
   state_push (data, object_info);
-  g_assert (state_peek (data) != NULL);
   object_info->tag.name = element_name;
 
   if (child_info)
     object_info->parent = (CommonInfo*)child_info;
+
+  g_markup_parse_context_get_position (context, &line, NULL);
+  line2 = GPOINTER_TO_INT (g_hash_table_lookup (data->object_ids, object_id));
+  if (line2 != 0)
+    {
+      g_set_error (error, GTK_BUILDER_ERROR,
+                   GTK_BUILDER_ERROR_DUPLICATE_ID,
+                   _("Duplicate object ID '%s' on line %d (previously on line %d)"),
+                   object_id, line, line2);
+      return;
+    }
+
+
+  g_hash_table_insert (data->object_ids, g_strdup (object_id), GINT_TO_POINTER (line));
 }
 
 static void
@@ -351,15 +441,12 @@ parse_child (ParserData   *data,
   object_info = state_peek_info (data, ObjectInfo);
   if (!object_info || strcmp (object_info->tag.name, "object") != 0)
     {
-      error_invalid_tag (data, element_name, "object", error);
-      if (object_info)
-	free_object_info (object_info);
+      error_invalid_tag (data, element_name, NULL, error);
       return;
     }
   
   child_info = g_slice_new0 (ChildInfo);
   state_push (data, child_info);
-  g_assert (state_peek (data) != NULL);
   child_info->tag.name = element_name;
   for (i = 0; names[i]; i++)
     {
@@ -373,7 +460,7 @@ parse_child (ParserData   *data,
 
   child_info->parent = (CommonInfo*)object_info;
 
-  object_info->object = builder_construct (data, object_info);
+  object_info->object = builder_construct (data, object_info, error);
 }
 
 static void
@@ -395,9 +482,15 @@ parse_property (ParserData   *data,
   gchar *name = NULL;
   gchar *context = NULL;
   gboolean translatable = FALSE;
+  ObjectInfo *object_info;
   int i;
 
-  g_assert (data->stack != NULL);
+  object_info = state_peek_info (data, ObjectInfo);
+  if (!object_info || strcmp (object_info->tag.name, "object") != 0)
+    {
+      error_invalid_tag (data, element_name, NULL, error);
+      return;
+    }
 
   for (i = 0; names[i] != NULL; i++)
     {
@@ -434,6 +527,7 @@ parse_property (ParserData   *data,
   info->name = name;
   info->translatable = translatable;
   info->context = context;
+  info->text = g_string_new ("");
   state_push (data, info);
 
   info->tag.name = element_name;
@@ -461,9 +555,15 @@ parse_signal (ParserData   *data,
   gboolean after = FALSE;
   gboolean swapped = FALSE;
   gboolean swapped_set = FALSE;
+  ObjectInfo *object_info;
   int i;
 
-  g_assert (data->stack != NULL);
+  object_info = state_peek_info (data, ObjectInfo);
+  if (!object_info || strcmp (object_info->tag.name, "object") != 0)
+    {
+      error_invalid_tag (data, element_name, NULL, error);
+      return;
+    }
 
   for (i = 0; names[i] != NULL; i++)
     {
@@ -534,6 +634,14 @@ _free_signal_info (SignalInfo *info,
   g_slice_free (SignalInfo, info);
 }
 
+void
+_free_requires_info (RequiresInfo *info,
+		     gpointer user_data)
+{
+  g_free (info->library);
+  g_slice_free (RequiresInfo, info);
+}
+
 static void
 parse_interface (ParserData   *data,
 		 const gchar  *element_name,
@@ -545,10 +653,25 @@ parse_interface (ParserData   *data,
 
   for (i = 0; names[i] != NULL; i++)
     {
-      if (strcmp (names[i], "domain") == 0 && !data->domain)
+      if (strcmp (names[i], "domain") == 0)
 	{
-	  data->domain = g_strdup (values[i]);
-	  break;
+	  if (data->domain)
+	    {
+	      if (strcmp (data->domain, values[i]) == 0)
+		continue;
+	      else
+		g_warning ("%s: interface domain '%s' overrides "
+			   "programically set domain '%s'",
+			   data->filename,
+			   values[i],
+			   data->domain
+			   );
+	      
+	      g_free (data->domain);
+	    }
+	  
+ 	  data->domain = g_strdup (values[i]);
+	  gtk_builder_set_translation_domain (data->builder, data->domain);
 	}
       else
 	error_invalid_attribute (data, "interface", names[i], error);
@@ -646,7 +769,7 @@ parse_custom (GMarkupParseContext *context,
 {
   CommonInfo* parent_info;
   GMarkupParser parser;
-  gpointer *subparser_data;
+  gpointer subparser_data;
   GObject *object;
   GObject *child;
 
@@ -658,8 +781,14 @@ parse_custom (GMarkupParseContext *context,
     {
       ObjectInfo* object_info = (ObjectInfo*)parent_info;
       if (!object_info->object)
-	object_info->object = _gtk_builder_construct (data->builder,
-						      object_info);
+	{
+	  object_info->properties = g_slist_reverse (object_info->properties);
+	  object_info->object = _gtk_builder_construct (data->builder,
+							object_info,
+							error);
+	  if (!object_info->object)
+	    return TRUE; /* A GError is already set */
+	}
       g_assert (object_info->object);
       object = object_info->object;
       child = NULL;
@@ -681,7 +810,7 @@ parse_custom (GMarkupParseContext *context,
 				       child,
 				       element_name,
 				       &parser,
-				       (gpointer*)&subparser_data))
+				       &subparser_data))
     return FALSE;
       
   data->subparser = create_subparser (object, child, element_name,
@@ -705,7 +834,7 @@ start_element (GMarkupParseContext *context,
   ParserData *data = (ParserData*)user_data;
 
 #ifdef GTK_ENABLE_DEBUG
-  if (gtk_debug_flags & GTK_DEBUG_BUILDER)
+  if (gtk_get_debug_flags () & GTK_DEBUG_BUILDER)
     {
       GString *tags = g_string_new ("");
       int i;
@@ -736,9 +865,16 @@ start_element (GMarkupParseContext *context,
     if (!subparser_start (context, element_name, names, values,
 			  data, error))
       return;
-  
-  if (strcmp (element_name, "object") == 0)
-    parse_object (data, element_name, names, values, error);
+
+  if (strcmp (element_name, "requires") == 0)
+    parse_requires (data, element_name, names, values, error);
+  else if (strcmp (element_name, "object") == 0)
+    parse_object (context, data, element_name, names, values, error);
+  else if (data->requested_objects && !data->inside_requested_object)
+    {
+      /* If outside a requested object, simply ignore this tag */
+      return;
+    }
   else if (strcmp (element_name, "child") == 0)
     parse_child (data, element_name, names, values, error);
   else if (strcmp (element_name, "property") == 0)
@@ -762,6 +898,21 @@ start_element (GMarkupParseContext *context,
 		   element_name);
 }
 
+gchar *
+_gtk_builder_parser_translate (const gchar *domain,
+			       const gchar *context,
+			       const gchar *text)
+{
+  const char *s;
+
+  if (context)
+    s = g_dpgettext2 (domain, context, text);
+  else
+    s = g_dgettext (domain, text);
+
+  return g_strdup (s);
+}
+
 /* Called for close tags </foo> */
 static void
 end_element (GMarkupParseContext *context,
@@ -779,19 +930,67 @@ end_element (GMarkupParseContext *context,
       return;
     }
 
-  if (strcmp (element_name, "object") == 0)
+  if (strcmp (element_name, "requires") == 0)
+    {
+      RequiresInfo *req_info = state_pop_info (data, RequiresInfo);
+
+      /* TODO: Allow third party widget developers to check thier
+       * required versions, possibly throw a signal allowing them
+       * to check thier library versions here.
+       */
+      if (!strcmp (req_info->library, "gtk+"))
+	{
+	  if (!GTK_CHECK_VERSION (req_info->major, req_info->minor, 0))
+	    g_set_error (error,
+			 GTK_BUILDER_ERROR,
+			 GTK_BUILDER_ERROR_VERSION_MISMATCH,
+			 "%s: required %s version %d.%d, current version is %d.%d",
+			 data->filename, req_info->library,
+			 req_info->major, req_info->minor,
+			 GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
+	}
+      _free_requires_info (req_info, NULL);
+    }
+  else if (strcmp (element_name, "interface") == 0)
+    {
+    }
+  else if (data->requested_objects && !data->inside_requested_object)
+    {
+      /* If outside a requested object, simply ignore this tag */
+      return;
+    }
+  else if (strcmp (element_name, "object") == 0)
     {
       ObjectInfo *object_info = state_pop_info (data, ObjectInfo);
       ChildInfo* child_info = state_peek_info (data, ChildInfo);
 
-      object_info->object = builder_construct (data, object_info);
+      if (data->requested_objects && data->inside_requested_object &&
+          (data->cur_object_level == data->requested_object_level))
+        {
+          GTK_NOTE (BUILDER, g_print ("requested object end found at level %d\n",
+                                      data->requested_object_level));
 
+          data->inside_requested_object = FALSE;
+        }
+
+      --data->cur_object_level;
+
+      g_assert (data->cur_object_level >= 0);
+
+      object_info->object = builder_construct (data, object_info, error);
+      if (!object_info->object)
+	{
+	  free_object_info (object_info);
+	  return;
+	}
       if (child_info)
         child_info->object = object_info->object;
 
       if (GTK_IS_BUILDABLE (object_info->object) &&
           GTK_BUILDABLE_GET_IFACE (object_info->object)->parser_finished)
         data->finalizers = g_slist_prepend (data->finalizers, object_info->object);
+      _gtk_builder_add_signals (data->builder, object_info->signals);
+
       free_object_info (object_info);
     }
   else if (strcmp (element_name, "property") == 0)
@@ -799,19 +998,24 @@ end_element (GMarkupParseContext *context,
       PropertyInfo *prop_info = state_pop_info (data, PropertyInfo);
       CommonInfo *info = state_peek_info (data, CommonInfo);
 
-      if (!prop_info->data)
-	{
-	  error_missing_property_value (data, error);
-	  free_property_info (prop_info);
-	  if (strcmp (info->tag.name, "object") == 0)
-	    free_object_info((ObjectInfo*)info);
-	  return;
-	}
-      
       /* Normal properties */
       if (strcmp (info->tag.name, "object") == 0)
         {
           ObjectInfo *object_info = (ObjectInfo*)info;
+
+          if (prop_info->translatable && prop_info->text->len)
+            {
+	      prop_info->data = _gtk_builder_parser_translate (data->domain,
+							       prop_info->context,
+							       prop_info->text->str);
+              g_string_free (prop_info->text, TRUE);
+            }
+          else
+            {
+              prop_info->data = g_string_free (prop_info->text, FALSE);
+              
+            }
+
           object_info->properties =
             g_slist_prepend (object_info->properties, prop_info);
         }
@@ -834,9 +1038,6 @@ end_element (GMarkupParseContext *context,
       object_info->signals =
         g_slist_prepend (object_info->signals, signal_info);
     }
-  else if (strcmp (element_name, "interface") == 0)
-    {
-    }
   else if (strcmp (element_name, "placeholder") == 0)
     {
     }
@@ -846,40 +1047,6 @@ end_element (GMarkupParseContext *context,
     }
 }
 
-/* This function is taken from gettext.h 
- * GNU gettext uses '\004' to separate context and msgid in .mo files.
- */
-static const char *
-dpgettext (const char *domain,
-           const char *msgctxt,
-           const char *msgid)
-{
-  size_t msgctxt_len = strlen (msgctxt) + 1;
-  size_t msgid_len = strlen (msgid) + 1;
-  const char *translation;
-  char* msg_ctxt_id;
-
-  msg_ctxt_id = g_alloca (msgctxt_len + msgid_len);
-  
-  memcpy (msg_ctxt_id, msgctxt, msgctxt_len - 1);
-  msg_ctxt_id[msgctxt_len - 1] = '\004';
-  memcpy (msg_ctxt_id + msgctxt_len, msgid, msgid_len);
-
-  translation = dgettext (domain, msg_ctxt_id);
-
-  if (translation == msg_ctxt_id) 
-    {
-      /* try the old way of doing message contexts, too */
-      msg_ctxt_id[msgctxt_len - 1] = '|';
-      translation = dgettext (domain, msg_ctxt_id);
-  
-      if (translation == msg_ctxt_id)
-        return msgid;
-    }
- 
-  return translation;
-}
-
 /* Called for character data */
 /* text is not nul-terminated */
 static void
@@ -894,9 +1061,13 @@ text (GMarkupParseContext *context,
 
   if (data->subparser && data->subparser->start)
     {
+      GError *tmp_error = NULL;
+      
       if (data->subparser->parser->text)
         data->subparser->parser->text (context, text, text_len,
-                                       data->subparser->data, error);
+                                       data->subparser->data, &tmp_error);
+      if (tmp_error)
+	g_propagate_error (error, tmp_error);
       return;
     }
 
@@ -910,24 +1081,27 @@ text (GMarkupParseContext *context,
     {
       PropertyInfo *prop_info = (PropertyInfo*)info;
 
-      /* text is not guaranteed to be null-terminated */
-      char *string = g_strndup (text, text_len);
-
-      if (prop_info->translatable && text_len)
-        {
-	  if (prop_info->context)
-            text = dpgettext (data->domain, prop_info->context, string);
-          else
-            text = dgettext (data->domain, string);
-
-	  prop_info->data = g_strdup (text);
-	  g_free (string);
-        }
-      else
-	prop_info->data = string;
+      g_string_append_len (prop_info->text, text, text_len);
     }
 }
 
+static void
+free_info (CommonInfo *info)
+{
+  if (strcmp (info->tag.name, "object") == 0) 
+    free_object_info ((ObjectInfo *)info);
+  else if (strcmp (info->tag.name, "child") == 0) 
+    free_child_info ((ChildInfo *)info);
+  else if (strcmp (info->tag.name, "property") == 0) 
+    free_property_info ((PropertyInfo *)info);
+  else if (strcmp (info->tag.name, "signal") == 0) 
+    _free_signal_info ((SignalInfo *)info, NULL);
+  else if (strcmp (info->tag.name, "requires") == 0) 
+    _free_requires_info ((RequiresInfo *)info, NULL);
+  else 
+    g_assert_not_reached ();
+}
+
 static const GMarkupParser parser = {
   start_element,
   end_element,
@@ -941,23 +1115,52 @@ _gtk_builder_parser_parse_buffer (GtkBuilder   *builder,
                                   const gchar  *filename,
                                   const gchar  *buffer,
                                   gsize         length,
+                                  gchar       **requested_objs,
                                   GError      **error)
 {
+  const gchar* domain;
   ParserData *data;
   GSList *l;
   
+  /* Store the original domain so that interface domain attribute can be
+   * applied for the builder and the original domain can be restored after
+   * parsing has finished. This allows subparsers to translate elements with
+   * gtk_builder_get_translation_domain() without breaking the ABI or API
+   */
+  domain = gtk_builder_get_translation_domain (builder);
+
   data = g_new0 (ParserData, 1);
   data->builder = builder;
   data->filename = filename;
-  data->domain = g_strdup (gtk_builder_get_translation_domain (builder));
+  data->domain = g_strdup (domain);
+  data->object_ids = g_hash_table_new_full (g_str_hash, g_str_equal,
+					    (GDestroyNotify)g_free, NULL);
 
-  data->ctx = g_markup_parse_context_new (
-                  &parser, G_MARKUP_TREAT_CDATA_AS_TEXT, data, NULL);
+  data->requested_objects = NULL;
+  if (requested_objs)
+    {
+      gint i;
+
+      data->inside_requested_object = FALSE;
+      for (i = 0; requested_objs[i]; ++i)
+        {
+          data->requested_objects = g_slist_prepend (data->requested_objects,
+                                                     g_strdup (requested_objs[i]));	
+        }
+    }
+  else
+    {
+      /* get all the objects */
+      data->inside_requested_object = TRUE;
+    }
+
+  data->ctx = g_markup_parse_context_new (&parser, 
+                                          G_MARKUP_TREAT_CDATA_AS_TEXT, 
+                                          data, NULL);
 
   if (!g_markup_parse_context_parse (data->ctx, buffer, length, error))
     goto out;
-  
-  gtk_builder_set_translation_domain (data->builder, data->domain);
+
   _gtk_builder_finish (builder);
 
   /* Custom parser_finished */
@@ -971,7 +1174,6 @@ _gtk_builder_parser_parse_buffer (GtkBuilder   *builder,
                                      sub->child,
                                      sub->tagname,
                                      sub->data);
-      free_subparser (sub);
     }
   
   /* Common parser_finished, for all created objects */
@@ -983,11 +1185,19 @@ _gtk_builder_parser_parse_buffer (GtkBuilder   *builder,
     }
 
  out:
-  g_markup_parse_context_free (data->ctx);
 
+  g_slist_foreach (data->stack, (GFunc)free_info, NULL);
   g_slist_free (data->stack);
+  g_slist_foreach (data->custom_finalizers, (GFunc)free_subparser, NULL);
   g_slist_free (data->custom_finalizers);
   g_slist_free (data->finalizers);
+  g_slist_foreach (data->requested_objects, (GFunc) g_free, NULL);
+  g_slist_free (data->requested_objects);
   g_free (data->domain);
+  g_hash_table_destroy (data->object_ids);
+  g_markup_parse_context_free (data->ctx);
   g_free (data);
+
+  /* restore the original domain */
+  gtk_builder_set_translation_domain (builder, domain);
 }
