@@ -40,11 +40,12 @@ G_BEGIN_DECLS
 
 typedef struct _GtkStyleContext GtkStyleContext;
 typedef struct _GtkStyleContextClass GtkStyleContextClass;
+typedef struct _GtkStyleContextPrivate GtkStyleContextPrivate;
 
 struct _GtkStyleContext
 {
   GObject parent_object;
-  gpointer priv;
+  GtkStyleContextPrivate *priv;
 };
 
 struct _GtkStyleContextClass
@@ -52,6 +53,12 @@ struct _GtkStyleContextClass
   GObjectClass parent_class;
 
   void (* changed) (GtkStyleContext *context);
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 /* Default set of properties that GtkStyleContext may contain */
@@ -78,31 +85,40 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_PROPERTY_FONT "font"
 
-/**
- * GTK_STYLE_PROPERTY_MARGIN:
- *
- * A property holding the rendered element's margin as a #GtkBorder. The
- * margin is defined as the spacing between the border of the element
- * and its surrounding elements.
- */
-#define GTK_STYLE_PROPERTY_MARGIN "margin"
-
 /**
  * GTK_STYLE_PROPERTY_PADDING:
  *
  * A property holding the rendered element's padding as a #GtkBorder. The
  * padding is defined as the spacing between the inner part of the element border
- * and its child.
+ * and its child. It's the innermost spacing property of the padding/border/margin
+ * series.
  */
 #define GTK_STYLE_PROPERTY_PADDING "padding"
 
 /**
  * GTK_STYLE_PROPERTY_BORDER_WIDTH:
  *
- * A property holding the rendered element's border width in pixels as a #gint.
+ * A property holding the rendered element's border width in pixels as
+ * a #GtkBorder. The border is the intermediary spacing property of the
+ * padding/border/margin series.
+ *
+ * gtk_render_frame() uses this property to find out the frame line width,
+ * so #GtkWidget<!-- -->s rendering frames may need to add up this padding when
+ * requesting size
  */
 #define GTK_STYLE_PROPERTY_BORDER_WIDTH "border-width"
 
+/**
+ * GTK_STYLE_PROPERTY_MARGIN:
+ *
+ * A property holding the rendered element's margin as a #GtkBorder. The
+ * margin is defined as the spacing between the border of the element
+ * and its surrounding elements. It is external to #GtkWidget<!-- -->s's
+ * size allocations, and the most external spacing property of the
+ * padding/border/margin series.
+ */
+#define GTK_STYLE_PROPERTY_MARGIN "margin"
+
 /**
  * GTK_STYLE_PROPERTY_BORDER_RADIUS:
  *
@@ -218,6 +234,13 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_CLASS_TOOLBAR "toolbar"
 
+/**
+ * GTK_STYLE_CLASS_PRIMARY_TOOLBAR:
+ *
+ * A CSS class to match primary toolbars.
+ */
+#define GTK_STYLE_CLASS_PRIMARY_TOOLBAR "primary-toolbar"
+
 /**
  * GTK_STYLE_CLASS_RADIO:
  *
@@ -260,6 +283,24 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_CLASS_SCALE "scale"
 
+/**
+ * GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE:
+ *
+ * A CSS class to match scale widgets with marks attached,
+ * all the marks are above for horizontal #GtkScale.
+ * left for vertical #GtkScale.
+ */
+#define GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE "scale-has-marks-above"
+
+/**
+ * GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW:
+ *
+ * A CSS class to match scale widgets with marks attached,
+ * all the marks are below for horizontal #GtkScale,
+ * right for vertical #GtkScale.
+ */
+#define GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW "scale-has-marks-below"
+
 /**
  * GTK_STYLE_CLASS_HEADER:
  *
@@ -337,6 +378,43 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_CLASS_VIEW "view"
 
+/**
+ * GTK_STYLE_CLASS_HIGHLIGHT:
+ *
+ * A CSS class defining a highlighted area, such as headings in
+ * assistants.
+ */
+#define GTK_STYLE_CLASS_HIGHLIGHT "highlight"
+
+/**
+ * GTK_STYLE_CLASS_FRAME:
+ *
+ * A CSS class defining a frame delimiting content, such as GtkFrame
+ * or the scrolled window frame around the scrollable area.
+ */
+#define GTK_STYLE_CLASS_FRAME "frame"
+
+/**
+ * GTK_STYLE_CLASS_DND:
+ *
+ * A CSS class for a drag-and-drop indicator
+ */
+#define GTK_STYLE_CLASS_DND "dnd"
+
+/**
+ * GTK_STYLE_CLASS_PANE_SEPARATOR:
+ *
+ * A CSS class for a pane separator, such as those in #GtkPaned.
+ */
+#define GTK_STYLE_CLASS_PANE_SEPARATOR "pane-separator"
+
+/**
+ * GTK_STYLE_CLASS_SEPARATOR:
+ *
+ * A CSS class for a separator.
+ */
+#define GTK_STYLE_CLASS_SEPARATOR "separator"
+
 /**
  * GTK_STYLE_CLASS_INFO:
  *
@@ -369,6 +447,20 @@ struct _GtkStyleContextClass
  */
 #define GTK_STYLE_CLASS_ERROR "error"
 
+/**
+ * GTK_STYLE_CLASS_HORIZONTAL:
+ *
+ * A widget class for horizontally layered widgets.
+ */
+#define GTK_STYLE_CLASS_HORIZONTAL "horizontal"
+
+/**
+ * GTK_STYLE_CLASS_VERTICAL:
+ *
+ * A widget class for vertically layered widgets.
+ */
+#define GTK_STYLE_CLASS_VERTICAL "vertical"
+
 
 /* Predefined set of widget regions */
 
@@ -473,7 +565,7 @@ void gtk_style_context_get_style          (GtkStyleContext *context,
                                            ...);
 
 GtkIconSet * gtk_style_context_lookup_icon_set (GtkStyleContext *context,
-						const gchar     *stock_id);
+                                                const gchar     *stock_id);
 GdkPixbuf  * gtk_icon_set_render_icon_pixbuf   (GtkIconSet      *icon_set,
                                                 GtkStyleContext *context,
                                                 GtkIconSize      size);
@@ -487,7 +579,7 @@ void             gtk_style_context_set_direction (GtkStyleContext  *context,
 GtkTextDirection gtk_style_context_get_direction (GtkStyleContext  *context);
 
 void             gtk_style_context_set_junction_sides (GtkStyleContext  *context,
-						       GtkJunctionSides  sides);
+                                                       GtkJunctionSides  sides);
 GtkJunctionSides gtk_style_context_get_junction_sides (GtkStyleContext  *context);
 
 gboolean gtk_style_context_lookup_color (GtkStyleContext *context,
@@ -499,6 +591,13 @@ void  gtk_style_context_notify_state_change (GtkStyleContext *context,
                                              gpointer         region_id,
                                              GtkStateType     state,
                                              gboolean         state_value);
+void  gtk_style_context_cancel_animations   (GtkStyleContext *context,
+                                             gpointer         region_id);
+void  gtk_style_context_scroll_animations   (GtkStyleContext *context,
+                                             GdkWindow       *window,
+                                             gint             dx,
+                                             gint             dy);
+
 void gtk_style_context_push_animatable_region (GtkStyleContext *context,
                                                gpointer         region_id);
 void gtk_style_context_pop_animatable_region  (GtkStyleContext *context);
@@ -513,10 +612,9 @@ void gtk_style_context_get_background_color (GtkStyleContext *context,
 void gtk_style_context_get_border_color     (GtkStyleContext *context,
                                              GtkStateFlags    state,
                                              GdkRGBA         *color);
-
-const PangoFontDescription * gtk_style_context_get_font (GtkStyleContext *context,
-                                                    GtkStateFlags    state);
-
+const PangoFontDescription *
+     gtk_style_context_get_font             (GtkStyleContext *context,
+                                             GtkStateFlags    state);
 void gtk_style_context_get_border           (GtkStyleContext *context,
                                              GtkStateFlags    state,
                                              GtkBorder       *border);
@@ -527,111 +625,101 @@ void gtk_style_context_get_margin           (GtkStyleContext *context,
                                              GtkStateFlags    state,
                                              GtkBorder       *margin);
 
-/* Semi-private API */
-const GValue * _gtk_style_context_peek_style_property (GtkStyleContext *context,
-                                                       GType            widget_type,
-                                                       GtkStateFlags    state,
-                                                       GParamSpec      *pspec);
-void           _gtk_style_context_invalidate_animation_areas (GtkStyleContext *context);
-void           _gtk_style_context_coalesce_animation_areas   (GtkStyleContext *context,
-							      GtkWidget       *widget);
+void gtk_style_context_invalidate           (GtkStyleContext *context);
+void gtk_style_context_reset_widgets        (GdkScreen       *screen);
 
-void gtk_style_context_invalidate (GtkStyleContext *context);
-void gtk_style_context_reset_widgets (GdkScreen *screen);
-
-void gtk_style_context_set_background (GtkStyleContext *context,
-                                       GdkWindow       *window);
+void gtk_style_context_set_background       (GtkStyleContext *context,
+                                             GdkWindow       *window);
 
 /* Paint methods */
-void gtk_render_check (GtkStyleContext *context,
-                       cairo_t         *cr,
-                       gdouble          x,
-                       gdouble          y,
-                       gdouble          width,
-                       gdouble          height);
-void gtk_render_option (GtkStyleContext *context,
-                        cairo_t         *cr,
-                        gdouble          x,
-                        gdouble          y,
-                        gdouble          width,
-                        gdouble          height);
-void gtk_render_arrow  (GtkStyleContext *context,
-                        cairo_t         *cr,
-                        gdouble          angle,
-                        gdouble          x,
-                        gdouble          y,
-                        gdouble          size);
-void gtk_render_background (GtkStyleContext *context,
-                            cairo_t         *cr,
-                            gdouble          x,
-                            gdouble          y,
-                            gdouble          width,
-                            gdouble          height);
-void gtk_render_frame  (GtkStyleContext *context,
-                        cairo_t         *cr,
-                        gdouble          x,
-                        gdouble          y,
-                        gdouble          width,
-                        gdouble          height);
-void gtk_render_expander (GtkStyleContext *context,
-                          cairo_t         *cr,
-                          gdouble          x,
-                          gdouble          y,
-                          gdouble          width,
-                          gdouble          height);
-void gtk_render_focus    (GtkStyleContext *context,
-                          cairo_t         *cr,
-                          gdouble          x,
-                          gdouble          y,
-                          gdouble          width,
-                          gdouble          height);
-void gtk_render_layout   (GtkStyleContext *context,
-                          cairo_t         *cr,
-                          gdouble          x,
-                          gdouble          y,
-                          PangoLayout     *layout);
-void gtk_render_line     (GtkStyleContext *context,
-                          cairo_t         *cr,
-                          gdouble          x0,
-                          gdouble          y0,
-                          gdouble          x1,
-                          gdouble          y1);
-void gtk_render_slider   (GtkStyleContext *context,
-                          cairo_t         *cr,
-                          gdouble          x,
-                          gdouble          y,
-                          gdouble          width,
-                          gdouble          height,
-                          GtkOrientation   orientation);
-void gtk_render_frame_gap (GtkStyleContext *context,
-                           cairo_t         *cr,
-                           gdouble          x,
-                           gdouble          y,
-                           gdouble          width,
-                           gdouble          height,
-                           GtkPositionType  gap_side,
-                           gdouble          xy0_gap,
-                           gdouble          xy1_gap);
-void gtk_render_extension (GtkStyleContext *context,
-                           cairo_t         *cr,
-                           gdouble          x,
-                           gdouble          y,
-                           gdouble          width,
-                           gdouble          height,
-                           GtkPositionType  gap_side);
-void gtk_render_handle    (GtkStyleContext *context,
-                           cairo_t         *cr,
-                           gdouble          x,
-                           gdouble          y,
-                           gdouble          width,
-                           gdouble          height);
-void gtk_render_activity  (GtkStyleContext *context,
-                           cairo_t         *cr,
-                           gdouble          x,
-                           gdouble          y,
-                           gdouble          width,
-                           gdouble          height);
-
+void        gtk_render_check       (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_option      (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_arrow       (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              angle,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              size);
+void        gtk_render_background  (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_frame       (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_expander    (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_focus       (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_layout      (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    PangoLayout         *layout);
+void        gtk_render_line        (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x0,
+                                    gdouble              y0,
+                                    gdouble              x1,
+                                    gdouble              y1);
+void        gtk_render_slider      (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height,
+                                    GtkOrientation       orientation);
+void        gtk_render_frame_gap   (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height,
+                                    GtkPositionType      gap_side,
+                                    gdouble              xy0_gap,
+                                    gdouble              xy1_gap);
+void        gtk_render_extension   (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height,
+                                    GtkPositionType      gap_side);
+void        gtk_render_handle      (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
+void        gtk_render_activity    (GtkStyleContext     *context,
+                                    cairo_t             *cr,
+                                    gdouble              x,
+                                    gdouble              y,
+                                    gdouble              width,
+                                    gdouble              height);
 GdkPixbuf * gtk_render_icon_pixbuf (GtkStyleContext     *context,
                                     const GtkIconSource *source,
                                     GtkIconSize          size);
